from django_filters import CharFilter
from django_filters import rest_framework as filters
from django.contrib.auth import get_user_model
from  django.db.models import JSONField


User = get_user_model()


class UserFilter(filters.FilterSet):
    class Meta:
        model = User
        fields = '__all__'
        filter_overrides = {
            JSONField: {
                'filter_class': CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'exact',
                },
            }
        }


import jwt
import datetime
from django.conf import settings

JWT_SALT = "ds()udsjo@jlsdosjf)wjd_#(#)$"


def create_token(payload):
    headers = {
        "type": "jwt",
        "alg": "HS256"
    }
    payload['exp'] = datetime.datetime.now() + datetime.timedelta(seconds=settings.USER_JWT_TIME)
    result = jwt.encode(payload=payload, key=JWT_SALT, algorithm="HS256", headers=headers)
    return result


def parse_payload(token):
    result = {"status": False, "data": None, "error": None}
    try:
        verified_payload = jwt.decode(token, JWT_SALT, algorithms="HS256", verify=True)
        result["status"] = True
        result['data'] = verified_payload
    except jwt.exceptions.ExpiredSignatureError:
        result['err'] = 'Token Expired'
    except jwt.DecodeError:
        result['err'] = 'Token Authentication Failed'
    except jwt.InvalidTokenError:
        result['err'] = 'Illegal Token'
    return result



from rest_framework_csv.renderers import CSVStreamingRenderer

def file_headers():
    return [
    ]

def data_header():
    return dict([
    ])


class FileRender(CSVStreamingRenderer):
    header = file_headers()
    labels = data_header()



from django.urls import path, re_path
from . import views

urlpatterns = [
    path(r'user/', views.UserViewSet.as_view({"get": "list", "post": "create"}), name="User List"),
    re_path(r'^user/(?P<pk>\d+)/$', views.UserViewSet.as_view({
        'get': 'retrieve',
        'delete': 'destroy'
    }), name="User List __1"),
]



import os

from django.core.wsgi import get_wsgi_application

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'bomiot.server.server.settings')
os.environ.setdefault('RUN_MAIN', 'true')

application = get_wsgi_application()



from rest_framework import viewsets
from rest_framework.response import Response
from rest_framework.exceptions import APIException

from .serializers import UserSerializer
from .filter import UserFilter
from rest_framework.filters import OrderingFilter
from django_filters.rest_framework import DjangoFilterBackend

from django.contrib.auth import get_user_model


User = get_user_model()


class UserViewSet(viewsets.ModelViewSet):
    """
        retrieve:
            Response a data list（get）

        list:
            Response a data list（all）

        create:
            Create a data line（post）

        delete:
            Delete a data line（delete)
    """
    filter_backends = [DjangoFilterBackend, OrderingFilter, ]
    ordering_fields = ['id', "created_time", "updated_time", ]
    filter_class = UserFilter

    def get_project(self):
        try:
            data_id = self.kwargs.get('pk')
            return data_id
        except:
            return None

    def get_queryset(self):
        data_id = self.get_project()
        if self.request.user:
            if data_id is None:
                return User.objects.filter(openid=self.request.auth.openid, is_delete=False)
            else:
                return User.objects.filter(openid=self.request.auth.openid, id=data_id, is_delete=False)
        else:
            return User.objects.none()

    def get_serializer_class(self):
        if self.action in ['list', 'retrieve', 'create', 'destroy']:
            return UserSerializer
        else:
            return self.http_method_not_allowed(request=self.request)

    def create(self, request, *args, **kwargs):
        data = self.request.data
        data['openid'] = self.request.auth.openid
        return Response({"detail": "Bin size does not exists or it has been changed"})


    def destroy(self, request, **kwargs):
        qs = self.get_object()
        if qs.openid != self.request.auth.openid:
            raise APIException({"detail": "Cannot delete data which not yours"})
        else:
            qs.is_delete = True
            qs.save()
            serializer = self.get_serializer(qs, many=False)
            headers = self.get_success_headers(serializer.data)
            return Response(serializer.data, status=200, headers=headers)